#include <cstdio>
#include <ctime>

#include "matrix.h"

/* Code taken from the GLIBC manual.
 *
 * Subtract the ‘struct timespec’ values X and Y,
 * storing the result in RESULT.
 * Return 1 if the difference is negative, otherwise 0.
 */
static int
timespec_subtract (struct timespec *result,
                   struct timespec *x,
                   struct timespec *y)
{
  /* Perform the carry for the later subtraction by updating y. */
  if (x->tv_nsec < y->tv_nsec) {
    int nsec = (y->tv_nsec - x->tv_nsec) / 1000000000 + 1;
    y->tv_nsec -= 1000000000 * nsec;
    y->tv_sec += nsec;
  }
  if (x->tv_nsec - y->tv_nsec > 1000000000) {
    int nsec = (x->tv_nsec - y->tv_nsec) / 1000000000;
    y->tv_nsec += 1000000000 * nsec;
    y->tv_sec -= nsec;
  }

  /* Compute the time remaining to wait.
     tv_nsec is certainly positive. */
  result->tv_sec = x->tv_sec - y->tv_sec;
  result->tv_nsec = x->tv_nsec - y->tv_nsec;

  /* Return 1 if result is negative. */
  return x->tv_sec < y->tv_sec;
}

int
main(int argc, char **argv)
{
  if (argc != 2)
    {
      fprintf(stderr, "usage: %s <filename>\n", argv[0]);
      return -1;
    }

  Matrix *A = matrix_load_matrix_market(argv[1]);
  if (!A)
    return -1;

  /* For debugging, can be removed when implementation is finished. */
  matrix_dump_nonzeros(A);


  struct timespec start_time;
  clock_gettime(CLOCK_REALTIME, &start_time);

  /* Perform LU factorization here */

  struct timespec end_time;
  clock_gettime(CLOCK_REALTIME, &end_time);


  struct timespec elapsed_time;
  timespec_subtract(&elapsed_time, &end_time, &start_time);

  double elapsed = (double)elapsed_time.tv_sec + (double)elapsed_time.tv_nsec / 1000000000.0;
  fprintf(stderr, "elapsed time: %f s\n", elapsed);

  matrix_free(A);

  return 0;
}
